#ifndef NEW_YORK_TRANSIT_LINE_CORE_H
#define NEW_YORK_TRANSIT_LINE_CORE_H

#include "templater.h"
#include <functional>

namespace nytl {
    /* ============== For parsing =============================*/
    struct ParsingContext {
        std::string text;
        size_t pos = 0;
        size_t column = 0;
        size_t line = 0;
    };

    constexpr int EOFVAL = -999;

    int peep(ParsingContext& ctx);

    char skip(ParsingContext& ctx);
    void skip(ParsingContext& ctx, char ch);

    void skipWhitespace(ParsingContext& ctx);

    void skipString(ParsingContext& ctx, const std::string& str);

    /* Returns empty if what is ahead of us is not name */
    std::string readName(ParsingContext& ctx);

    /* Returns empty if what is ahead of us is not uint */
    std::string readUint(ParsingContext& ctx);

    std::vector<std::string> splitIntoLines(const std::string& str);
    std::string concatenateLines(const std::vector<std::string>& lines);
    void one_part_update_min_start_wsp_non_empty(const std::string& str, bool is_first, size_t& min);
    std::string one_part_cut_excess_tab(const std::string& str, bool is_first, size_t cut);

    void parse_bare_file(const std::string& filename, const std::string& content,
        global_elem_set_t& result);
    void parse_special_file(const std::string& filename, const std::string& content,
        global_elem_set_t& result);

    /* =================== For rendering ====================*/
    struct LocalVarValue {
        bool is_json = false;
        std::string EL_name;
        const json::JSON* JSON_subval = NULL;
    };

    /* No new JSON object will ever be created, I have one root json argument,
     * all the other json variables are subtrees of it */
    LocalVarValue rendering_core_execute_expression(const global_elem_set_t& global_elems,
        const std::vector<LocalVarValue>& local_vars, const json::JSON& expr);

    std::string rendering_core(const std::string& entry_func, const std::vector<json::JSON>& entry_arguments,
        const global_elem_set_t& elem_ns, const std::function<std::string(std::string)>& escape);
}

#endif
